JOB_MANAGER_ABI = [
  { "inputs": [], "stateMutability": "nonpayable", "type": "constructor" },
  { "inputs": [], "name": "AccessControlBadConfirmation", "type": "error" },
  {
    "inputs": [
      { "internalType": "address", "name": "account", "type": "address" },
      { "internalType": "bytes32", "name": "neededRole", "type": "bytes32" },
    ],
    "name": "AccessControlUnauthorizedAccount",
    "type": "error",
  },
  {
    "inputs": [{ "internalType": "address", "name": "target", "type": "address" }],
    "name": "AddressEmptyCode",
    "type": "error",
  },
  {
    "inputs": [
      { "internalType": "address", "name": "implementation", "type": "address" },
    ],
    "name": "ERC1967InvalidImplementation",
    "type": "error",
  },
  { "inputs": [], "name": "ERC1967NonPayable", "type": "error" },
  { "inputs": [], "name": "FailedInnerCall", "type": "error" },
  { "inputs": [], "name": "InvalidInitialization", "type": "error" },
  { "inputs": [], "name": "NotInitializing", "type": "error" },
  { "inputs": [], "name": "ReentrancyGuardReentrantCall", "type": "error" },
  { "inputs": [], "name": "UUPSUnauthorizedCallContext", "type": "error" },
  {
    "inputs": [{ "internalType": "bytes32", "name": "slot", "type": "bytes32" }],
    "name": "UUPSUnsupportedProxiableUUID",
    "type": "error",
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": True,
        "internalType": "uint256",
        "name": "jobId",
        "type": "uint256",
      },
      {
        "indexed": False,
        "internalType": "uint256",
        "name": "newBudget",
        "type": "uint256",
      },
    ],
    "name": "BudgetSet",
    "type": "event",
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": False,
        "internalType": "uint64",
        "name": "version",
        "type": "uint64",
      },
    ],
    "name": "Initialized",
    "type": "event",
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": True,
        "internalType": "uint256",
        "name": "jobId",
        "type": "uint256",
      },
      {
        "indexed": True,
        "internalType": "uint256",
        "name": "accountId",
        "type": "uint256",
      },
      {
        "indexed": True,
        "internalType": "address",
        "name": "client",
        "type": "address",
      },
      {
        "indexed": False,
        "internalType": "address",
        "name": "provider",
        "type": "address",
      },
      {
        "indexed": False,
        "internalType": "address",
        "name": "evaluator",
        "type": "address",
      },
      {
        "indexed": False,
        "internalType": "uint256",
        "name": "expiredAt",
        "type": "uint256",
      },
    ],
    "name": "JobCreated",
    "type": "event",
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": True,
        "internalType": "uint256",
        "name": "jobId",
        "type": "uint256",
      },
      {
        "indexed": True,
        "internalType": "address",
        "name": "paymentToken",
        "type": "address",
      },
      {
        "indexed": False,
        "internalType": "uint256",
        "name": "newBudget",
        "type": "uint256",
      },
    ],
    "name": "JobPaymentTokenSet",
    "type": "event",
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": True,
        "internalType": "uint256",
        "name": "jobId",
        "type": "uint256",
      },
      {
        "indexed": False,
        "internalType": "enum ACPTypes.JobPhase",
        "name": "oldPhase",
        "type": "uint8",
      },
      {
        "indexed": False,
        "internalType": "enum ACPTypes.JobPhase",
        "name": "newPhase",
        "type": "uint8",
      },
    ],
    "name": "JobPhaseUpdated",
    "type": "event",
  },
  {
    "anonymous": False,
    "inputs": [
      { "indexed": True, "internalType": "bytes32", "name": "role", "type": "bytes32" },
      {
        "indexed": True,
        "internalType": "bytes32",
        "name": "previousAdminRole",
        "type": "bytes32",
      },
      {
        "indexed": True,
        "internalType": "bytes32",
        "name": "newAdminRole",
        "type": "bytes32",
      },
    ],
    "name": "RoleAdminChanged",
    "type": "event",
  },
  {
    "anonymous": False,
    "inputs": [
      { "indexed": True, "internalType": "bytes32", "name": "role", "type": "bytes32" },
      {
        "indexed": True,
        "internalType": "address",
        "name": "account",
        "type": "address",
      },
      {
        "indexed": True,
        "internalType": "address",
        "name": "sender",
        "type": "address",
      },
    ],
    "name": "RoleGranted",
    "type": "event",
  },
  {
    "anonymous": False,
    "inputs": [
      { "indexed": True, "internalType": "bytes32", "name": "role", "type": "bytes32" },
      {
        "indexed": True,
        "internalType": "address",
        "name": "account",
        "type": "address",
      },
      {
        "indexed": True,
        "internalType": "address",
        "name": "sender",
        "type": "address",
      },
    ],
    "name": "RoleRevoked",
    "type": "event",
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": True,
        "internalType": "address",
        "name": "implementation",
        "type": "address",
      },
    ],
    "name": "Upgraded",
    "type": "event",
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": True,
        "internalType": "uint256",
        "name": "jobId",
        "type": "uint256",
      },
    ],
    "name": "X402PaymentReceived",
    "type": "event",
  },
  {
    "inputs": [],
    "name": "ACP_CONTRACT_ROLE",
    "outputs": [{ "internalType": "bytes32", "name": "", "type": "bytes32" }],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [],
    "name": "ADMIN_ROLE",
    "outputs": [{ "internalType": "bytes32", "name": "", "type": "bytes32" }],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [],
    "name": "DEFAULT_ADMIN_ROLE",
    "outputs": [{ "internalType": "bytes32", "name": "", "type": "bytes32" }],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [],
    "name": "MEMO_MANAGER_ROLE",
    "outputs": [{ "internalType": "bytes32", "name": "", "type": "bytes32" }],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [],
    "name": "UPGRADE_INTERFACE_VERSION",
    "outputs": [{ "internalType": "string", "name": "", "type": "string" }],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [],
    "name": "X402_MANAGER_ROLE",
    "outputs": [{ "internalType": "bytes32", "name": "", "type": "bytes32" }],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "", "type": "uint256" },
      { "internalType": "uint256", "name": "", "type": "uint256" },
    ],
    "name": "accountJobs",
    "outputs": [{ "internalType": "uint256", "name": "", "type": "uint256" }],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [],
    "name": "acpContract",
    "outputs": [{ "internalType": "address", "name": "", "type": "address" }],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "jobId", "type": "uint256" },
      { "internalType": "address", "name": "user", "type": "address" },
    ],
    "name": "canModifyJob",
    "outputs": [{ "internalType": "bool", "name": "", "type": "bool" }],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [{ "internalType": "uint256", "name": "jobId", "type": "uint256" }],
    "name": "confirmX402PaymentReceived",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "accountId", "type": "uint256" },
      { "internalType": "address", "name": "client", "type": "address" },
      { "internalType": "address", "name": "provider", "type": "address" },
      { "internalType": "address", "name": "evaluator", "type": "address" },
      { "internalType": "address", "name": "creator", "type": "address" },
      { "internalType": "uint256", "name": "budget", "type": "uint256" },
      {
        "internalType": "contract IERC20",
        "name": "paymentToken",
        "type": "address",
      },
      { "internalType": "uint256", "name": "expiredAt", "type": "uint256" },
    ],
    "name": "createJob",
    "outputs": [{ "internalType": "uint256", "name": "jobId", "type": "uint256" }],
    "stateMutability": "nonpayable",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "accountId", "type": "uint256" },
      { "internalType": "address", "name": "client", "type": "address" },
      { "internalType": "address", "name": "provider", "type": "address" },
      { "internalType": "address", "name": "evaluator", "type": "address" },
      { "internalType": "address", "name": "creator", "type": "address" },
      { "internalType": "uint256", "name": "budget", "type": "uint256" },
      {
        "internalType": "contract IERC20",
        "name": "paymentToken",
        "type": "address",
      },
      { "internalType": "uint256", "name": "expiredAt", "type": "uint256" },
    ],
    "name": "createJobWithX402",
    "outputs": [{ "internalType": "uint256", "name": "", "type": "uint256" }],
    "stateMutability": "nonpayable",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "accountId", "type": "uint256" },
      { "internalType": "uint256", "name": "offset", "type": "uint256" },
      { "internalType": "uint256", "name": "limit", "type": "uint256" },
    ],
    "name": "getAccountJobs",
    "outputs": [
      {
        "components": [
          { "internalType": "uint256", "name": "id", "type": "uint256" },
          { "internalType": "uint256", "name": "accountId", "type": "uint256" },
          { "internalType": "address", "name": "client", "type": "address" },
          { "internalType": "address", "name": "provider", "type": "address" },
          { "internalType": "address", "name": "evaluator", "type": "address" },
          { "internalType": "address", "name": "creator", "type": "address" },
          { "internalType": "uint256", "name": "budget", "type": "uint256" },
          {
            "internalType": "contract IERC20",
            "name": "jobPaymentToken",
            "type": "address",
          },
          {
            "internalType": "enum ACPTypes.JobPhase",
            "name": "phase",
            "type": "uint8",
          },
          { "internalType": "uint256", "name": "expiredAt", "type": "uint256" },
          { "internalType": "uint256", "name": "createdAt", "type": "uint256" },
          { "internalType": "uint256", "name": "memoCount", "type": "uint256" },
          { "internalType": "string", "name": "metadata", "type": "string" },
          { "internalType": "uint256", "name": "amountClaimed", "type": "uint256" },
        ],
        "internalType": "struct ACPTypes.Job[]",
        "name": "jobArray",
        "type": "tuple[]",
      },
      { "internalType": "uint256", "name": "total", "type": "uint256" },
    ],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [{ "internalType": "uint256", "name": "jobId", "type": "uint256" }],
    "name": "getJob",
    "outputs": [
      {
        "components": [
          { "internalType": "uint256", "name": "id", "type": "uint256" },
          { "internalType": "uint256", "name": "accountId", "type": "uint256" },
          { "internalType": "address", "name": "client", "type": "address" },
          { "internalType": "address", "name": "provider", "type": "address" },
          { "internalType": "address", "name": "evaluator", "type": "address" },
          { "internalType": "address", "name": "creator", "type": "address" },
          { "internalType": "uint256", "name": "budget", "type": "uint256" },
          {
            "internalType": "contract IERC20",
            "name": "jobPaymentToken",
            "type": "address",
          },
          {
            "internalType": "enum ACPTypes.JobPhase",
            "name": "phase",
            "type": "uint8",
          },
          { "internalType": "uint256", "name": "expiredAt", "type": "uint256" },
          { "internalType": "uint256", "name": "createdAt", "type": "uint256" },
          { "internalType": "uint256", "name": "memoCount", "type": "uint256" },
          { "internalType": "string", "name": "metadata", "type": "string" },
          { "internalType": "uint256", "name": "amountClaimed", "type": "uint256" },
        ],
        "internalType": "struct ACPTypes.Job",
        "name": "",
        "type": "tuple",
      },
    ],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [{ "internalType": "bytes32", "name": "role", "type": "bytes32" }],
    "name": "getRoleAdmin",
    "outputs": [{ "internalType": "bytes32", "name": "", "type": "bytes32" }],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "address", "name": "assignee", "type": "address" },
      { "internalType": "uint256", "name": "offset", "type": "uint256" },
      { "internalType": "uint256", "name": "limit", "type": "uint256" },
    ],
    "name": "getUserJobs",
    "outputs": [
      {
        "components": [
          { "internalType": "uint256", "name": "id", "type": "uint256" },
          { "internalType": "uint256", "name": "accountId", "type": "uint256" },
          { "internalType": "address", "name": "client", "type": "address" },
          { "internalType": "address", "name": "provider", "type": "address" },
          { "internalType": "address", "name": "evaluator", "type": "address" },
          { "internalType": "address", "name": "creator", "type": "address" },
          { "internalType": "uint256", "name": "budget", "type": "uint256" },
          {
            "internalType": "contract IERC20",
            "name": "jobPaymentToken",
            "type": "address",
          },
          {
            "internalType": "enum ACPTypes.JobPhase",
            "name": "phase",
            "type": "uint8",
          },
          { "internalType": "uint256", "name": "expiredAt", "type": "uint256" },
          { "internalType": "uint256", "name": "createdAt", "type": "uint256" },
          { "internalType": "uint256", "name": "memoCount", "type": "uint256" },
          { "internalType": "string", "name": "metadata", "type": "string" },
          { "internalType": "uint256", "name": "amountClaimed", "type": "uint256" },
        ],
        "internalType": "struct ACPTypes.Job[]",
        "name": "jobArray",
        "type": "tuple[]",
      },
      { "internalType": "uint256", "name": "total", "type": "uint256" },
    ],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [{ "internalType": "uint256", "name": "jobId", "type": "uint256" }],
    "name": "getX402PaymentDetails",
    "outputs": [
      {
        "components": [
          { "internalType": "bool", "name": "isX402", "type": "bool" },
          { "internalType": "bool", "name": "isBudgetReceived", "type": "bool" },
        ],
        "internalType": "struct ACPTypes.X402PaymentDetail",
        "name": "",
        "type": "tuple",
      },
    ],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "bytes32", "name": "role", "type": "bytes32" },
      { "internalType": "address", "name": "account", "type": "address" },
    ],
    "name": "grantRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "bytes32", "name": "role", "type": "bytes32" },
      { "internalType": "address", "name": "account", "type": "address" },
    ],
    "name": "hasRole",
    "outputs": [{ "internalType": "bool", "name": "", "type": "bool" }],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [{ "internalType": "uint256", "name": "jobId", "type": "uint256" }],
    "name": "incrementMemoCount",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "address", "name": "acpContract_", "type": "address" },
    ],
    "name": "initialize",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
  },
  {
    "inputs": [],
    "name": "jobCounter",
    "outputs": [{ "internalType": "uint256", "name": "", "type": "uint256" }],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [{ "internalType": "uint256", "name": "", "type": "uint256" }],
    "name": "jobs",
    "outputs": [
      { "internalType": "uint256", "name": "id", "type": "uint256" },
      { "internalType": "uint256", "name": "accountId", "type": "uint256" },
      { "internalType": "address", "name": "client", "type": "address" },
      { "internalType": "address", "name": "provider", "type": "address" },
      { "internalType": "address", "name": "evaluator", "type": "address" },
      { "internalType": "address", "name": "creator", "type": "address" },
      { "internalType": "uint256", "name": "budget", "type": "uint256" },
      {
        "internalType": "contract IERC20",
        "name": "jobPaymentToken",
        "type": "address",
      },
      { "internalType": "enum ACPTypes.JobPhase", "name": "phase", "type": "uint8" },
      { "internalType": "uint256", "name": "expiredAt", "type": "uint256" },
      { "internalType": "uint256", "name": "createdAt", "type": "uint256" },
      { "internalType": "uint256", "name": "memoCount", "type": "uint256" },
      { "internalType": "string", "name": "metadata", "type": "string" },
      { "internalType": "uint256", "name": "amountClaimed", "type": "uint256" },
    ],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [],
    "name": "proxiableUUID",
    "outputs": [{ "internalType": "bytes32", "name": "", "type": "bytes32" }],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "bytes32", "name": "role", "type": "bytes32" },
      { "internalType": "address", "name": "callerConfirmation", "type": "address" },
    ],
    "name": "renounceRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "bytes32", "name": "role", "type": "bytes32" },
      { "internalType": "address", "name": "account", "type": "address" },
    ],
    "name": "revokeRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "jobId", "type": "uint256" },
      { "internalType": "uint256", "name": "budget", "type": "uint256" },
      {
        "internalType": "contract IERC20",
        "name": "paymentToken",
        "type": "address",
      },
    ],
    "name": "setJobBudget",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "x402PaymentTokenAddress",
        "type": "address",
      },
    ],
    "name": "setX402PaymentToken",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
  },
  {
    "inputs": [{ "internalType": "bytes4", "name": "interfaceId", "type": "bytes4" }],
    "name": "supportsInterface",
    "outputs": [{ "internalType": "bool", "name": "", "type": "bool" }],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "address", "name": "acpContract_", "type": "address" },
    ],
    "name": "updateContracts",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "jobId", "type": "uint256" },
      {
        "internalType": "enum ACPTypes.JobPhase",
        "name": "newPhase",
        "type": "uint8",
      },
    ],
    "name": "updateJobPhase",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "address", "name": "newImplementation", "type": "address" },
      { "internalType": "bytes", "name": "data", "type": "bytes" },
    ],
    "name": "upgradeToAndCall",
    "outputs": [],
    "stateMutability": "payable",
    "type": "function",
  },
  {
    "inputs": [
      { "internalType": "address", "name": "", "type": "address" },
      { "internalType": "uint256", "name": "", "type": "uint256" },
    ],
    "name": "userJobs",
    "outputs": [{ "internalType": "uint256", "name": "", "type": "uint256" }],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [{ "internalType": "uint256", "name": "jobId", "type": "uint256" }],
    "name": "x402PaymentDetails",
    "outputs": [
      { "internalType": "bool", "name": "isX402", "type": "bool" },
      { "internalType": "bool", "name": "isBudgetReceived", "type": "bool" },
    ],
    "stateMutability": "view",
    "type": "function",
  },
  {
    "inputs": [],
    "name": "x402PaymentToken",
    "outputs": [{ "internalType": "contract IERC20", "name": "", "type": "address" }],
    "stateMutability": "view",
    "type": "function",
  },
];


initial_acp_v2_abi = [
  { "inputs": [], "stateMutability": "nonpayable", "type": "constructor" },
  { "inputs": [], "name": "AccessControlBadConfirmation", "type": "error" },
  {
    "inputs": [
      { "internalType": "address", "name": "account", "type": "address" },
      { "internalType": "bytes32", "name": "neededRole", "type": "bytes32" }
    ],
    "name": "AccessControlUnauthorizedAccount",
    "type": "error"
  },
  {
    "inputs": [{ "internalType": "address", "name": "target", "type": "address" }],
    "name": "AddressEmptyCode",
    "type": "error"
  },
  {
    "inputs": [{ "internalType": "address", "name": "account", "type": "address" }],
    "name": "AddressInsufficientBalance",
    "type": "error"
  },
  {
    "inputs": [
      { "internalType": "address", "name": "implementation", "type": "address" }
    ],
    "name": "ERC1967InvalidImplementation",
    "type": "error"
  },
  { "inputs": [], "name": "ERC1967NonPayable", "type": "error" },
  { "inputs": [], "name": "EnforcedPause", "type": "error" },
  { "inputs": [], "name": "ExpectedPause", "type": "error" },
  { "inputs": [], "name": "FailedInnerCall", "type": "error" },
  { "inputs": [], "name": "InvalidInitialization", "type": "error" },
  { "inputs": [], "name": "NotInitializing", "type": "error" },
  { "inputs": [], "name": "ReentrancyGuardReentrantCall", "type": "error" },
  {
    "inputs": [{ "internalType": "address", "name": "token", "type": "address" }],
    "name": "SafeERC20FailedOperation",
    "type": "error"
  },
  { "inputs": [], "name": "UUPSUnauthorizedCallContext", "type": "error" },
  {
    "inputs": [{ "internalType": "bytes32", "name": "slot", "type": "bytes32" }],
    "name": "UUPSUnsupportedProxiableUUID",
    "type": "error"
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": True,
        "internalType": "uint256",
        "name": "accountId",
        "type": "uint256"
      },
      {
        "indexed": True,
        "internalType": "address",
        "name": "client",
        "type": "address"
      },
      {
        "indexed": True,
        "internalType": "address",
        "name": "provider",
        "type": "address"
      }
    ],
    "name": "AccountCreated",
    "type": "event"
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": True,
        "internalType": "uint256",
        "name": "accountId",
        "type": "uint256"
      },
      { "indexed": False, "internalType": "bool", "name": "isActive", "type": "bool" }
    ],
    "name": "AccountStatusUpdated",
    "type": "event"
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": False,
        "internalType": "uint256",
        "name": "jobId",
        "type": "uint256"
      },
      {
        "indexed": True,
        "internalType": "address",
        "name": "evaluator",
        "type": "address"
      },
      {
        "indexed": False,
        "internalType": "uint256",
        "name": "evaluatorFee",
        "type": "uint256"
      }
    ],
    "name": "ClaimedEvaluatorFee",
    "type": "event"
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": False,
        "internalType": "uint256",
        "name": "jobId",
        "type": "uint256"
      },
      {
        "indexed": True             ,
        "internalType": "address",
        "name": "provider",
        "type": "address"
      },
      {
        "indexed": False,
        "internalType": "uint256",
        "name": "providerFee",
        "type": "uint256"
      }
    ],
    "name": "ClaimedProviderFee",
    "type": "event"
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": False,
        "internalType": "uint64",
        "name": "version",
        "type": "uint64"
      }
    ],
    "name": "Initialized",
    "type": "event"
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": True,
        "internalType": "string",
        "name": "moduleType",
        "type": "string"
      },
      {
        "indexed": True,
        "internalType": "address",
        "name": "oldModule",
        "type": "address"
      },
      {
        "indexed": True,
        "internalType": "address",
        "name": "newModule",
        "type": "address"
      }
    ],
    "name": "ModuleUpdated",
    "type": "event"
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": False,
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "Paused",
    "type": "event"
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": False,
        "internalType": "uint256",
        "name": "jobId",
        "type": "uint256"
      },
      {
        "indexed": True,
        "internalType": "address",
        "name": "client",
        "type": "address"
      },
      {
        "indexed": False,
        "internalType": "uint256",
        "name": "amount",
        "type": "uint256"
      }
    ],
    "name": "RefundedBudget",
    "type": "event"
  },
  {
    "anonymous": False,
    "inputs": [
      { "indexed": True, "internalType": "bytes32", "name": "role", "type": "bytes32" },
      {
        "indexed": True,
        "internalType": "bytes32",
        "name": "previousAdminRole",
        "type": "bytes32"
      },
      {
        "indexed": True,
        "internalType": "bytes32",
        "name": "newAdminRole",
        "type": "bytes32"
      }
    ],
    "name": "RoleAdminChanged",
    "type": "event"
  },
  {
    "anonymous": False,
    "inputs": [
      { "indexed": True, "internalType": "bytes32", "name": "role", "type": "bytes32" },
      {
        "indexed": True,
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "indexed": True,
        "internalType": "address",
        "name": "sender",
        "type": "address"
      }
    ],
    "name": "RoleGranted",
    "type": "event"
  },
  {
    "anonymous": False,
    "inputs": [
      { "indexed": True, "internalType": "bytes32", "name": "role", "type": "bytes32" },
      {
        "indexed": True,
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "indexed": True,
        "internalType": "address",
        "name": "sender",
        "type": "address"
      }
    ],
    "name": "RoleRevoked",
    "type": "event"
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": False,
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "Unpaused",
    "type": "event"
  },
  {
    "anonymous": False,
    "inputs": [
      {
        "indexed": True,
        "internalType": "address",
        "name": "implementation",
        "type": "address"
      }
    ],
    "name": "Upgraded",
    "type": "event"
  },
  {
    "inputs": [],
    "name": "ADMIN_ROLE",
    "outputs": [{ "internalType": "bytes32", "name": "", "type": "bytes32" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "DEFAULT_ADMIN_ROLE",
    "outputs": [{ "internalType": "bytes32", "name": "", "type": "bytes32" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "MODULE_MANAGER_ROLE",
    "outputs": [{ "internalType": "bytes32", "name": "", "type": "bytes32" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "OPERATOR_ROLE",
    "outputs": [{ "internalType": "bytes32", "name": "", "type": "bytes32" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "UPGRADE_INTERFACE_VERSION",
    "outputs": [{ "internalType": "string", "name": "", "type": "string" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "accountManager",
    "outputs": [
      { "internalType": "contract IAccountManager", "name": "", "type": "address" }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "address", "name": "account", "type": "address" },
      { "internalType": "uint256", "name": "jobId", "type": "uint256" }
    ],
    "name": "canSign",
    "outputs": [{ "internalType": "bool", "name": "", "type": "bool" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [{ "internalType": "uint256", "name": "jobId", "type": "uint256" }],
    "name": "claimBudget",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "address", "name": "provider", "type": "address" },
      { "internalType": "string", "name": "metadata", "type": "string" }
    ],
    "name": "createAccount",
    "outputs": [{ "internalType": "uint256", "name": "", "type": "uint256" }],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "jobId", "type": "uint256" },
      { "internalType": "string", "name": "content", "type": "string" },
      { "internalType": "address", "name": "token", "type": "address" },
      { "internalType": "uint256", "name": "amount", "type": "uint256" },
      { "internalType": "address", "name": "recipient", "type": "address" },
      { "internalType": "uint256", "name": "feeAmount", "type": "uint256" },
      { "internalType": "enum ACPTypes.FeeType", "name": "feeType", "type": "uint8" },
      {
        "internalType": "enum ACPTypes.MemoType",
        "name": "memoType",
        "type": "uint8"
      },
      { "internalType": "uint256", "name": "expiredAt", "type": "uint256" },
      { "internalType": "bool", "name": "isSecured", "type": "bool" },
      {
        "internalType": "enum ACPTypes.JobPhase",
        "name": "nextPhase",
        "type": "uint8"
      },
      { "internalType": "uint32", "name": "lzDstEid", "type": "uint32" }
    ],
    "name": "createCrossChainPayableMemo",
    "outputs": [{ "internalType": "uint256", "name": "", "type": "uint256" }],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "address", "name": "provider", "type": "address" },
      { "internalType": "address", "name": "evaluator", "type": "address" },
      { "internalType": "uint256", "name": "expiredAt", "type": "uint256" },
      { "internalType": "address", "name": "paymentToken", "type": "address" },
      { "internalType": "uint256", "name": "budget", "type": "uint256" },
      { "internalType": "string", "name": "metadata", "type": "string" }
    ],
    "name": "createJob",
    "outputs": [{ "internalType": "uint256", "name": "", "type": "uint256" }],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "accountId", "type": "uint256" },
      { "internalType": "address", "name": "evaluator", "type": "address" },
      { "internalType": "uint256", "name": "budget", "type": "uint256" },
      { "internalType": "address", "name": "paymentToken", "type": "address" },
      { "internalType": "uint256", "name": "expiredAt", "type": "uint256" }
    ],
    "name": "createJobWithAccount",
    "outputs": [{ "internalType": "uint256", "name": "", "type": "uint256" }],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "jobId", "type": "uint256" },
      { "internalType": "string", "name": "content", "type": "string" },
      {
        "internalType": "enum ACPTypes.MemoType",
        "name": "memoType",
        "type": "uint8"
      },
      { "internalType": "bool", "name": "isSecured", "type": "bool" },
      {
        "internalType": "enum ACPTypes.JobPhase",
        "name": "nextPhase",
        "type": "uint8"
      }
    ],
    "name": "createMemo",
    "outputs": [{ "internalType": "uint256", "name": "", "type": "uint256" }],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "jobId", "type": "uint256" },
      { "internalType": "string", "name": "content", "type": "string" },
      { "internalType": "address", "name": "token", "type": "address" },
      { "internalType": "uint256", "name": "amount", "type": "uint256" },
      { "internalType": "address", "name": "recipient", "type": "address" },
      { "internalType": "uint256", "name": "feeAmount", "type": "uint256" },
      { "internalType": "enum ACPTypes.FeeType", "name": "feeType", "type": "uint8" },
      {
        "internalType": "enum ACPTypes.MemoType",
        "name": "memoType",
        "type": "uint8"
      },
      { "internalType": "uint256", "name": "expiredAt", "type": "uint256" },
      { "internalType": "bool", "name": "isSecured", "type": "bool" },
      {
        "internalType": "enum ACPTypes.JobPhase",
        "name": "nextPhase",
        "type": "uint8"
      }
    ],
    "name": "createPayableMemo",
    "outputs": [{ "internalType": "uint256", "name": "", "type": "uint256" }],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "address", "name": "provider", "type": "address" },
      { "internalType": "address", "name": "evaluator", "type": "address" },
      { "internalType": "uint256", "name": "expiredAt", "type": "uint256" },
      { "internalType": "address", "name": "paymentToken", "type": "address" },
      { "internalType": "uint256", "name": "budget", "type": "uint256" },
      { "internalType": "string", "name": "metadata", "type": "string" }
    ],
    "name": "createX402Job",
    "outputs": [{ "internalType": "uint256", "name": "", "type": "uint256" }],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "accountId", "type": "uint256" },
      { "internalType": "address", "name": "evaluator", "type": "address" },
      { "internalType": "uint256", "name": "budget", "type": "uint256" },
      { "internalType": "address", "name": "paymentToken", "type": "address" },
      { "internalType": "uint256", "name": "expiredAt", "type": "uint256" }
    ],
    "name": "createX402JobWithAccount",
    "outputs": [{ "internalType": "uint256", "name": "", "type": "uint256" }],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "defaultPaymentToken",
    "outputs": [{ "internalType": "contract IERC20", "name": "", "type": "address" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "address", "name": "token", "type": "address" },
      { "internalType": "uint256", "name": "amount", "type": "uint256" }
    ],
    "name": "emergencyWithdraw",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "evaluatorFeeBP",
    "outputs": [{ "internalType": "uint256", "name": "", "type": "uint256" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [{ "internalType": "uint256", "name": "accountId", "type": "uint256" }],
    "name": "getAccount",
    "outputs": [
      {
        "components": [
          { "internalType": "uint256", "name": "id", "type": "uint256" },
          { "internalType": "address", "name": "client", "type": "address" },
          { "internalType": "address", "name": "provider", "type": "address" },
          { "internalType": "uint256", "name": "createdAt", "type": "uint256" },
          { "internalType": "string", "name": "metadata", "type": "string" },
          { "internalType": "uint256", "name": "jobCount", "type": "uint256" },
          {
            "internalType": "uint256",
            "name": "completedJobCount",
            "type": "uint256"
          },
          { "internalType": "bool", "name": "isActive", "type": "bool" }
        ],
        "internalType": "struct ACPTypes.Account",
        "name": "",
        "type": "tuple"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "jobId", "type": "uint256" },
      { "internalType": "uint256", "name": "offset", "type": "uint256" },
      { "internalType": "uint256", "name": "limit", "type": "uint256" }
    ],
    "name": "getAllMemos",
    "outputs": [
      {
        "components": [
          { "internalType": "uint256", "name": "id", "type": "uint256" },
          { "internalType": "uint256", "name": "jobId", "type": "uint256" },
          { "internalType": "address", "name": "sender", "type": "address" },
          { "internalType": "string", "name": "content", "type": "string" },
          {
            "internalType": "enum ACPTypes.MemoType",
            "name": "memoType",
            "type": "uint8"
          },
          { "internalType": "uint256", "name": "createdAt", "type": "uint256" },
          { "internalType": "bool", "name": "isApproved", "type": "bool" },
          { "internalType": "address", "name": "approvedBy", "type": "address" },
          { "internalType": "uint256", "name": "approvedAt", "type": "uint256" },
          { "internalType": "bool", "name": "requiresApproval", "type": "bool" },
          { "internalType": "string", "name": "metadata", "type": "string" },
          { "internalType": "bool", "name": "isSecured", "type": "bool" },
          {
            "internalType": "enum ACPTypes.JobPhase",
            "name": "nextPhase",
            "type": "uint8"
          },
          { "internalType": "uint256", "name": "expiredAt", "type": "uint256" },
          {
            "internalType": "enum ACPTypes.MemoState",
            "name": "state",
            "type": "uint8"
          }
        ],
        "internalType": "struct ACPTypes.Memo[]",
        "name": "memos",
        "type": "tuple[]"
      },
      { "internalType": "uint256", "name": "total", "type": "uint256" }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "jobId", "type": "uint256" },
      {
        "internalType": "enum ACPTypes.MemoType",
        "name": "memoType",
        "type": "uint8"
      },
      { "internalType": "uint256", "name": "offset", "type": "uint256" },
      { "internalType": "uint256", "name": "limit", "type": "uint256" }
    ],
    "name": "getMemosForMemoType",
    "outputs": [
      {
        "components": [
          { "internalType": "uint256", "name": "id", "type": "uint256" },
          { "internalType": "uint256", "name": "jobId", "type": "uint256" },
          { "internalType": "address", "name": "sender", "type": "address" },
          { "internalType": "string", "name": "content", "type": "string" },
          {
            "internalType": "enum ACPTypes.MemoType",
            "name": "memoType",
            "type": "uint8"
          },
          { "internalType": "uint256", "name": "createdAt", "type": "uint256" },
          { "internalType": "bool", "name": "isApproved", "type": "bool" },
          { "internalType": "address", "name": "approvedBy", "type": "address" },
          { "internalType": "uint256", "name": "approvedAt", "type": "uint256" },
          { "internalType": "bool", "name": "requiresApproval", "type": "bool" },
          { "internalType": "string", "name": "metadata", "type": "string" },
          { "internalType": "bool", "name": "isSecured", "type": "bool" },
          {
            "internalType": "enum ACPTypes.JobPhase",
            "name": "nextPhase",
            "type": "uint8"
          },
          { "internalType": "uint256", "name": "expiredAt", "type": "uint256" },
          {
            "internalType": "enum ACPTypes.MemoState",
            "name": "state",
            "type": "uint8"
          }
        ],
        "internalType": "struct ACPTypes.Memo[]",
        "name": "memos",
        "type": "tuple[]"
      },
      { "internalType": "uint256", "name": "total", "type": "uint256" }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "jobId", "type": "uint256" },
      { "internalType": "enum ACPTypes.JobPhase", "name": "phase", "type": "uint8" },
      { "internalType": "uint256", "name": "offset", "type": "uint256" },
      { "internalType": "uint256", "name": "limit", "type": "uint256" }
    ],
    "name": "getMemosForPhaseType",
    "outputs": [
      {
        "components": [
          { "internalType": "uint256", "name": "id", "type": "uint256" },
          { "internalType": "uint256", "name": "jobId", "type": "uint256" },
          { "internalType": "address", "name": "sender", "type": "address" },
          { "internalType": "string", "name": "content", "type": "string" },
          {
            "internalType": "enum ACPTypes.MemoType",
            "name": "memoType",
            "type": "uint8"
          },
          { "internalType": "uint256", "name": "createdAt", "type": "uint256" },
          { "internalType": "bool", "name": "isApproved", "type": "bool" },
          { "internalType": "address", "name": "approvedBy", "type": "address" },
          { "internalType": "uint256", "name": "approvedAt", "type": "uint256" },
          { "internalType": "bool", "name": "requiresApproval", "type": "bool" },
          { "internalType": "string", "name": "metadata", "type": "string" },
          { "internalType": "bool", "name": "isSecured", "type": "bool" },
          {
            "internalType": "enum ACPTypes.JobPhase",
            "name": "nextPhase",
            "type": "uint8"
          },
          { "internalType": "uint256", "name": "expiredAt", "type": "uint256" },
          {
            "internalType": "enum ACPTypes.MemoState",
            "name": "state",
            "type": "uint8"
          }
        ],
        "internalType": "struct ACPTypes.Memo[]",
        "name": "memos",
        "type": "tuple[]"
      },
      { "internalType": "uint256", "name": "total", "type": "uint256" }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getPhases",
    "outputs": [{ "internalType": "string[7]", "name": "", "type": "string[7]" }],
    "stateMutability": "pure",
    "type": "function"
  },
  {
    "inputs": [{ "internalType": "bytes32", "name": "role", "type": "bytes32" }],
    "name": "getRoleAdmin",
    "outputs": [{ "internalType": "bytes32", "name": "", "type": "bytes32" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "bytes32", "name": "role", "type": "bytes32" },
      { "internalType": "address", "name": "account", "type": "address" }
    ],
    "name": "grantRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "bytes32", "name": "role", "type": "bytes32" },
      { "internalType": "address", "name": "account", "type": "address" }
    ],
    "name": "hasRole",
    "outputs": [{ "internalType": "bool", "name": "", "type": "bool" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "defaultPaymentToken_",
        "type": "address"
      },
      { "internalType": "uint256", "name": "platformFeeBP_", "type": "uint256" },
      { "internalType": "address", "name": "platformTreasury_", "type": "address" },
      { "internalType": "uint256", "name": "evaluatorFeeBP_", "type": "uint256" }
    ],
    "name": "initialize",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "jobId", "type": "uint256" },
      { "internalType": "address", "name": "account", "type": "address" }
    ],
    "name": "isJobEvaluator",
    "outputs": [{ "internalType": "bool", "name": "", "type": "bool" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "jobManager",
    "outputs": [
      { "internalType": "contract IJobManager", "name": "", "type": "address" }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "memoManager",
    "outputs": [
      { "internalType": "contract IMemoManager", "name": "", "type": "address" }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "pause",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "paused",
    "outputs": [{ "internalType": "bool", "name": "", "type": "bool" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "paymentManager",
    "outputs": [
      { "internalType": "contract IPaymentManager", "name": "", "type": "address" }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "platformFeeBP",
    "outputs": [{ "internalType": "uint256", "name": "", "type": "uint256" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "platformTreasury",
    "outputs": [{ "internalType": "address", "name": "", "type": "address" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "proxiableUUID",
    "outputs": [{ "internalType": "bytes32", "name": "", "type": "bytes32" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "bytes32", "name": "role", "type": "bytes32" },
      { "internalType": "address", "name": "callerConfirmation", "type": "address" }
    ],
    "name": "renounceRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "bytes32", "name": "role", "type": "bytes32" },
      { "internalType": "address", "name": "account", "type": "address" }
    ],
    "name": "revokeRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "jobId", "type": "uint256" },
      { "internalType": "uint256", "name": "amount", "type": "uint256" }
    ],
    "name": "setBudget",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "jobId", "type": "uint256" },
      { "internalType": "uint256", "name": "amount", "type": "uint256" },
      { "internalType": "address", "name": "paymentToken", "type": "address" }
    ],
    "name": "setBudgetWithPaymentToken",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "memoId", "type": "uint256" },
      { "internalType": "bool", "name": "isApproved", "type": "bool" },
      { "internalType": "string", "name": "reason", "type": "string" }
    ],
    "name": "signMemo",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [{ "internalType": "bytes4", "name": "interfaceId", "type": "bytes4" }],
    "name": "supportsInterface",
    "outputs": [{ "internalType": "bool", "name": "", "type": "bool" }],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "unpause",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "accountId", "type": "uint256" },
      { "internalType": "string", "name": "metadata", "type": "string" }
    ],
    "name": "updateAccountMetadata",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "evaluatorFeeBP_", "type": "uint256" }
    ],
    "name": "updateEvaluatorFee",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "string", "name": "moduleType", "type": "string" },
      { "internalType": "address", "name": "moduleAddress", "type": "address" }
    ],
    "name": "updateModule",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "uint256", "name": "platformFeeBP_", "type": "uint256" },
      { "internalType": "address", "name": "platformTreasury_", "type": "address" },
      { "internalType": "uint256", "name": "evaluatorFeeBP_", "type": "uint256" }
    ],
    "name": "updatePlatformConfig",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      { "internalType": "address", "name": "newImplementation", "type": "address" },
      { "internalType": "bytes", "name": "data", "type": "bytes" }
    ],
    "name": "upgradeToAndCall",
    "outputs": [],
    "stateMutability": "payable",
    "type": "function"
  }
]

custom_abi = [
    {
        "anonymous": False,
        "inputs": [
            {
                "indexed": True,
                "internalType": "uint256",
                "name": "jobId",
                "type": "uint256",
            },
            {
                "indexed": True,
                "internalType": "uint256",
                "name": "accountId",
                "type": "uint256",
            },
            {
                "indexed": True,
                "internalType": "address",
                "name": "client",
                "type": "address",
            },
            {
                "indexed": False,
                "internalType": "address",
                "name": "provider",
                "type": "address",
            },
            {
                "indexed": False,
                "internalType": "address",
                "name": "evaluator",
                "type": "address",
            },
            {
                "indexed": False,
                "internalType": "uint256",
                "name": "expiredAt",
                "type": "uint256",
            },
        ],
        "name": "JobCreated",
        "type": "event",
    }
]



ACP_V2_ABI = initial_acp_v2_abi + custom_abi



import json
import logging
import secrets
from typing import Dict, Any, List, Optional

from eth_account import Account
from eth_account.messages import encode_typed_data
from eth_utils.crypto import keccak
from web3 import Web3

from virtuals_acp.abis.job_manager import JOB_MANAGER_ABI
from virtuals_acp.abis.memo_manager import MEMO_MANAGER_ABI
from virtuals_acp.alchemy import AlchemyAccountKit
from virtuals_acp.configs.configs import ACPContractConfig, BASE_MAINNET_CONFIG_V2
from virtuals_acp.constants import SINGLE_SIGNER_VALIDATION_MODULE_ADDRESS
from virtuals_acp.contract_clients.base_contract_client import BaseAcpContractClient
from virtuals_acp.exceptions import ACPError
from virtuals_acp.models import AcpJobX402PaymentDetails, OffChainJob, OperationPayload, X402PayableRequest, X402PayableRequirements, X402Payment
from virtuals_acp.x402 import ACPX402

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s [%(levelname)s] %(name)s: %(message)s",
)
logger = logging.getLogger("ContractClientV2")

class ACPContractClientV2(BaseAcpContractClient):
    def __init__(
        self,
        agent_wallet_address: str,
        wallet_private_key: str,
        entity_id: int,
        config: ACPContractConfig = BASE_MAINNET_CONFIG_V2,
    ):
        super().__init__(agent_wallet_address, config)

        self.account = Account.from_key(wallet_private_key)
        self.entity_id = entity_id
        self.alchemy_kit = AlchemyAccountKit(
            config=config,
            agent_wallet_address=agent_wallet_address,
            entity_id=entity_id,
            owner_account=self.account,
            chain_id=config.chain_id,
            chains=config.chains,
        )
        self.w3 = Web3(Web3.HTTPProvider(config.rpc_url))
        self.x402 = ACPX402(config, self.account, self.w3, self.agent_wallet_address, self.entity_id)

        self.public_clients = {}
        for c in config.chains:
            self.public_clients[c.chain_id] = Web3(Web3.HTTPProvider(c.rpc_url))

        def multicall_read(
            w3: Web3, contract_address: str, abi: list[Dict[str, Any]], calls: list[str]
        ):
            contract = w3.eth.contract(
                address=Web3.to_checksum_address(contract_address), abi=abi
            )
            results = []
            for fn_name in calls:
                fn = getattr(contract.functions, fn_name)
                results.append(fn().call())
            return results

        calls = ["jobManager", "memoManager", "accountManager"]
        job_manager, memo_manager, account_manager = multicall_read(
            self.w3, config.contract_address, config.abi, calls
        )

        if not all([job_manager, memo_manager, account_manager]):
            raise ACPError("Failed to fetch sub-manager contract addresses")

        self.job_manager_address = Web3.to_checksum_address(job_manager)
        self.job_manager_contract = self.w3.eth.contract(
            address=self.job_manager_address, abi=JOB_MANAGER_ABI
        )

        self.memo_manager_address = Web3.to_checksum_address(memo_manager)
        self.memo_manager_contract = self.w3.eth.contract(
            address=self.memo_manager_address, abi=MEMO_MANAGER_ABI
        )

        self.validate_session_key_on_chain(self.account.address, self.entity_id)

        logger.info(
            "\nConnected to ACP:\n%s",
            json.dumps(
                {
                    "agent_wallet_address": agent_wallet_address,
                    "whitelisted_wallet_address": self.account.address,
                    "entity_id": self.entity_id,
                },
                indent=2
            )
        )
        
    def get_acp_version(self) -> str:
        return "2"

    def _get_random_nonce(self, bits: int = 152) -> int:
        """Generate a random bigint nonce."""
        bytes_len = bits // 8
        random_bytes = secrets.token_bytes(bytes_len)
        return int.from_bytes(random_bytes, byteorder="big")

    def handle_operation(self, trx_data: List[OperationPayload], chain_id: Optional[int] = None) -> Dict[str, Any]:
        return self.alchemy_kit.handle_user_operation(trx_data, chain_id=chain_id)

    def get_job_id(
        self, response: Dict[str, Any], client_address: str, provider_address: str
    ) -> int:
        logs: List[Dict[str, Any]] = response.get("receipts", [])[0].get("logs", [])

        decoded_create_job_logs = [
            self.contract.events.JobCreated().process_log(
                {
                    "topics": log["topics"],
                    "data": log["data"],
                    "address": log["address"],
                    "logIndex": 0,
                    "transactionIndex": 0,
                    "transactionHash": "0x0000",
                    "blockHash": "0x0000",
                    "blockNumber": 0,
                }
            )
            for log in logs
            if log["topics"][0] == self.job_created_event_signature_hex
        ]

        if len(decoded_create_job_logs) == 0:
            raise Exception("No logs found for JobCreated event")

        created_job_log = next(
            (
                log
                for log in decoded_create_job_logs
                if log["args"]["provider"] == provider_address
                and log["args"]["client"] == client_address
            ),
            None,
        )

        if not created_job_log:
            raise Exception(
                "No logs found for JobCreated event with provider and client addresses"
            )

        return int(created_job_log["args"]["jobId"])
    
    def update_job_x402_nonce(self, job_id: int, nonce: str) -> OffChainJob:
        """Update job X402 nonce."""
        try:
            return self.x402.update_job_nonce(job_id, nonce)
        except Exception as e:
            raise ACPError("Failed to update job X402 nonce", e)

    def generate_x402_payment(
        self, 
        payable_request: X402PayableRequest, 
        requirements: X402PayableRequirements
    ) -> X402Payment:
        """Generate X402 payment."""
        try:
            return self.x402.generate_payment(payable_request, requirements)
        except Exception as e:
            raise ACPError("Failed to generate X402 payment", e)
        
    def perform_x402_request(self, url: str, version: str, budget: Optional[str] = None, signature: Optional[str] = None) -> Dict[str, Any]:
        try:
            return self.x402.perform_request(url, version, budget, signature)
        except Exception as e:
            raise ACPError("Failed to perform X402 request", e)

    def get_x402_payment_details(self, job_id: int) -> AcpJobX402PaymentDetails:
        """Get X402 payment details for a job."""
        try:
            from virtuals_acp.abis.job_manager import JOB_MANAGER_ABI

            x402_config = self.config.x402_config
            if not x402_config or not getattr(x402_config, "url", None):
                return AcpJobX402PaymentDetails(is_x402=False, is_budget_received=False)

            # Use the job manager address & JOB_MANAGER_ABI directly
            job_manager_address = Web3.to_checksum_address(self.job_manager_address)
            if not job_manager_address:
                raise ACPError("Job manager address not provided in config.")

            job_manager_contract = self.w3.eth.contract(
                address=job_manager_address,
                abi=JOB_MANAGER_ABI,
            )

            result = job_manager_contract.functions.x402PaymentDetails(job_id).call()

            return AcpJobX402PaymentDetails(
                is_x402=result[0], is_budget_received=result[1]
            )
        except Exception as e:
            raise ACPError("Failed to get X402 payment details", e)

    def get_asset_manager_address(self) -> str:
        return self.memo_manager_contract.functions.assetManager().call()

    def sign_typed_data(self, typed_data: dict[str, Any]) -> str:
        encoded = encode_typed_data(full_message=typed_data)
        typed_data_hash = keccak(b"\x19\x01" + encoded.header + encoded.body)

        replay_safe_typed_data = {
            "domain": {
                "chainId": self.config.chain_id,
                "verifyingContract": SINGLE_SIGNER_VALIDATION_MODULE_ADDRESS,
                "salt": "0x" + "00" * 12 + self.agent_wallet_address[2:],
            },
            "types": {"ReplaySafeHash": [{"name": "hash", "type": "bytes32"}]},
            "message": {"hash": "0x" + typed_data_hash.hex()},
            "primaryType": "ReplaySafeHash",
        }

        signable = encode_typed_data(full_message=replay_safe_typed_data)
        signed = self.account.sign_message(signable)
        raw_signature = signed.signature.hex()
        return f"0x{self._pack_1271_eoa_signature(raw_signature)}"

    def _pack_1271_eoa_signature(self, validation_signature: str) -> str:
        if validation_signature.startswith("0x"):
            validation_signature = validation_signature[2:]

        prefix = b"\x00"
        entity_id_bytes = self.entity_id.to_bytes(4, "big")
        separator = b"\xff"
        eoa_type = b"\x00"
        sig_bytes = bytes.fromhex(validation_signature)

        packed = prefix + entity_id_bytes + separator + eoa_type + sig_bytes
        return packed.hex()



from typing import Union

from web3 import Web3
from virtuals_acp.abis.erc20_abi import ERC20_ABI

# TODO: implement wrapper methods in base_contract_client

def getERC20Balance(
    public_client: Web3,
    contract_address: str,
    wallet_address: str,
) -> int:
    erc20_contract_instance = public_client.eth.contract(
        address=contract_address, abi=ERC20_ABI
    )
    
    balance = erc20_contract_instance.functions.balanceOf(wallet_address).call()
    
    return balance

def getERC20Allowance(
    public_client: Web3,
    contract_address: str,
    wallet_address: str,
    spender_address: str,
) -> int:
    erc20_contract_instance = public_client.eth.contract(
        address=contract_address, abi=ERC20_ABI
    )
    
    allowance = erc20_contract_instance.functions.allowance(wallet_address, spender_address).call()
    
    return allowance

def getERC20Symbol(
    public_client: Web3,
    contract_address: str,
) -> str:
    erc20_contract_instance = public_client.eth.contract(
        address=contract_address, abi=ERC20_ABI
    )
    
    symbol = erc20_contract_instance.functions.symbol().call()
    
    return symbol

def getERC20Decimals(
    public_client: Web3,
    contract_address: str,
) -> int:
    erc20_contract_instance = public_client.eth.contract(
        address=contract_address, abi=ERC20_ABI
    )
    
    decimals = erc20_contract_instance.functions.decimals().call()
    
    return decimals